function [feat prms] = glac_on_grid(Grad, prms)
%
% GLAC feature extraction on spatial bins
%
%  [feat prms] = glac(Grad, prms)
%
% Input
%   Grad - Gradient data matrix [height x width x 2(dx,dy)]
%   prms - Parameters [struct]
%          .rvecs   - Displacement (rx,ry) [{[0 1; 1 1; 1 0; 1 -1]}]
%                      or scalar (r) for r*[0 1; 1 1; 1 0; 1 -1]
%          .ztol    - Tolerance for zero gradients [{0.001}] 
%          .nbin    - Number of orientation bins   [{9}]
%          .reflect - true  for calculating direction in half-region (0~180) [{false}|true]
%                     false for whole-region (0~360)
%          .nsbins  - Number of spatial bins (nwbin, nhbin)  [{[1 1]}]
%                     or scalar value (nsbin) for [nsbin nsbin]
%          .gamma   - Parameter of position weight, exp[-gamma*{(x-mu_x)^2+(y-mu_y)^2}],  [{0}]
%
% Output
%   feat - Feature vector [nbin*nwbin*nhbin + nr*(nbin^2*nwbin*nhbin) x 1]
%   prms - Parameters [struct]
%          in which missing fields are filled with default values.
%
% Reference:
% [1] T. Kobayashi and N. Otsu, 
%     "Image Feature Extraction Using Gradient Local Auto-correlations", ECCV 2008.
%

%- Parameters -%
def_rvecs  = [0 1; 1 1; 1 0; 1 -1];
def_nsbins = [1,1];
if ~exist('prms', 'var') || isempty(prms), prms = struct; end
prms = parseparam(prms, 'rvecs',def_rvecs, 'ztol',0.001, 'nbin',9, 'reflect',false, 'nsbins',def_nsbins, 'gamma',0);
if isscalar(prms.rvecs),  prms.rvecs  = prms.rvecs*def_rvecs; end
if isscalar(prms.nsbins), prms.nsbins = prms.nsbins*def_nsbins; end

%- GLAC on grid -%
feat = glac_on_grid_mex(Grad, prms.rvecs, prms.ztol, prms.nbin, prms.reflect, prms.nsbins, prms.gamma);
