#ifndef GLAC_HPP
#define GLAC_HPP

#include <math.h>

#define MIN(a,b) ((a)>(b) ? (b):(a))
#define ABS(a) ((a)>0 ? (a):(-(a)))

#ifndef M_PI
#define M_PI 3.141592
#endif

/*---------------------------------------------------------------*
 *  Orientation coding
 *     e.g., #bins = 8
 *           5 6 7
 *            \|/
 *          4-- --0       +--->x
 *            /|\         |
 *           3 2 1        v
 *                        y
 * Input:
 *  num  - Number of pixels
 *  grad - Gradient vectors [num x 2 (dx, dy)]
 *  nbin - Number of orientation bins
 *  ztol - Tolerance for zero gradient
 *	reflect - 1 for calculating direction in half-region (0~180)
 *            0 for whole-region (0~360)
 *
 * Output:
 * 	output - Orientation code data [num x 3]
 * 	         (magnitude, bin-index, weight),....
 *---------------------------------------------------------------*/
void orientation_code(double* output, double* grad, int num, int nbin, double ztol, int reflect)
{
	double radian2bin = nbin/(2*M_PI);

	double* xgrad = &grad[0];
	double* ygrad = &grad[num];

	for(int i = 0; i < num; i++){
		double* mag = &output[3*i];   //magnitude
		double* ind = &output[3*i+1]; //bin index
		double* wei = &output[3*i+2]; //weight for the bin

		double x = xgrad[i];
		double y = ygrad[i];
		*mag = sqrt(x*x+y*y);

		/*- Tiny edge is ignored -*/
		if(*mag < ztol){
			*mag = -1;
			*ind = -1;
			continue;
		}

		/*- Compute angle -*/
		double theta = 0;
		if (reflect > 0){
			//HALFREGION (0~180)
			double x2 = x*x-y*y;  //r*r*cos
			double y2 = 2*x*y;    //r*r*sin	
			theta = atan2(y2,x2);
		}else{
			//WHOLEREGION (0~360)
			theta = atan2(y,x);
		}
		if(theta < 0) theta += 2*M_PI;

		/*- Coding -*/
		double dindex = theta * radian2bin;
		int     index = (int)floor(dindex);
		if(index == nbin){
			*ind = 0;
			*wei = 1;
		}else{
			*ind = index;
			*wei = 1 + index - dindex;
		}
	}
}

/*---------------------------------------------------------------*
 *   Gradient Local Auto-Correlation  (GLAC)
 *
 * Input:
 * 	idata - Orientation code data [height x width x 3]
 * 	         (magnitude, bin-index, weight),....
 *  nbin  - Number of orientation bins   [scalar]
 *  rvecs - Displacement vectors (rx,ry) [nr x 2]
 *
 * Output:
 *   ofeat - Feature vector [nbin + nr*nbin^2 x 1]
 *---------------------------------------------------------------*/
void calc_glac(double* ofeat, double* idata, int height, int width, int nbin, int* rvecs, int nr)
{
	double* feat0th = &ofeat[0];
	double* feat1st = &ofeat[nbin];

	for(int x = 0; x < width; x++)
		for(int y = 0; y < height; y++){
			int i = y+x*height;

			/*- Gradient Magnitude -*/
			double mag1 = idata[i*3];
			if( mag1 < 0) continue;

			/*- Orientation index -*/
			int ind1  = (int)idata[i*3+1];
			int ind1n = (ind1+1)%nbin; //next bin

			/*- Weight for the index -*/
			double wei1  = idata[i*3+2];
			double wei1n = 1-wei1; //weight for the next bin

			feat0th[ind1 ] += mag1 * wei1;
			feat0th[ind1n] += mag1 * wei1n;

			for(int l = 0; l < nr; l++){
				int dx = x+rvecs[l];
				int dy = y+rvecs[l+nr];

				if( dy >= 0 && dy < height && dx >= 0 && dx < width ){
					int j = dy+dx*height;

					/*- Gradient Magnitude -*/
					double mag2 = idata[j*3];
					if( mag2 < 0) continue; //tiny edge is ignored

					/*- Orientation index -*/
					int ind2  = (int)idata[j*3+1];
					int ind2n = (ind2+1)%nbin; //next bin

					/*- Weight for the index -*/
					double wei2  = idata[j*3+2];
					double wei2n = 1-wei2; //weight for the next bin

					/*- Weight for the pair -*/
					double mag = MIN(mag1, mag2);

					/*- Bilinear voting -*/
					double* feat1st_ = &feat1st[l*nbin*nbin];
					feat1st_[ind1  + ind2  * nbin] += mag * wei1  * wei2;
					feat1st_[ind1n + ind2  * nbin] += mag * wei1n * wei2;
					feat1st_[ind1  + ind2n * nbin] += mag * wei1  * wei2n;
					feat1st_[ind1n + ind2n * nbin] += mag * wei1n * wei2n;
				}
			}
		}
}

/*--------------------------------------------------------------------------*
 *   Gradient Local Auto-Correlation (GLAC) on spatial bins
 *
 * Input:
 * 	idata - Orientation code data [height x width x 3]
 * 	         (magnitude, bin-index, weight),....
 *  nbin  - Number of orientation bins   [scalar]
 *  rvecs - Displacement vectors (rx,ry) [nr x 2]
 *  nhbin,nwbin - Number of spatial bins  [scalar]
 *  gamma - Parameter of position weight, exp[-gamma*{(x-mu_x)^2+(y-mu_y)^2}]
 *
 * Output:
 *  ofeat - Feature vector [nbin*nwbin*nhbin + nr*(nbin^2*nwbin*nhbin) x 1]
 *--------------------------------------------------------------------------*/
void calc_glac_on_grid(double* ofeat, double* idata, int height,int width, int nbin, int* rvecs, int nr, int nhbin, int nwbin, double gamma)
{
	double pixel2wbin = nwbin/(double)width;
	double pixel2hbin = nhbin/(double)height;

	double centerx = 0.5*width;
	double centery = 0.5*height;

	double* feat0th = &ofeat[0];
	double* feat1st = &ofeat[nbin*nwbin*nhbin];

	int dim1st = nbin*nbin;
	int dim1st_per_rvec = dim1st*nwbin*nhbin;
	
	for(int x = 0; x < width; x++){
		for(int y = 0; y < height; y++){
			int i = y+x*height;

			/* Gradient Magnitude */
			double mag1 = idata[i*3];
			if( mag1 < 0) continue;

			/*- Orientation index -*/
			int ind1  = (int)idata[i*3+1];
			int ind1n = (ind1+1)%nbin; //next bin

			/*- Weight for the index -*/
			double wei1  = idata[i*3+2];
			double wei1n = 1-wei1; //weight for the next bin

			/*- x-pos in spatial bins -*/
			double realx = pixel2wbin*x;
			int     binx = (int)floor(realx-0.5);
			double rbinx = realx - 0.5 - binx;

			/*- y-pos in spatial bins -*/
			double realy = pixel2hbin*y;
			int     biny = (int)floor(realy-0.5);
			double rbiny = realy - 0.5 - biny;

			/*- Position weighting -*/
			double w = exp( -gamma*((x-centerx)*(x-centerx)+(y-centery)*(y-centery)) );
			
			/*- Trilinear voting -*/
			for(int ix = 0; ix < 2; ix++)
				for(int iy = 0; iy < 2; iy++)
					if( binx + ix >= 0 && binx + ix < nwbin && biny + iy >= 0 && biny + iy < nhbin ){
						double weight = w * mag1 * ABS(1 - ix - rbinx) * ABS(1 - iy - rbiny);
						int offset = nbin * (binx+ix+(biny+iy)*nwbin);
						feat0th[offset + ind1 ] += weight * wei1;
						feat0th[offset + ind1n] += weight * wei1n;
					}
			
			for(int l = 0; l < nr; l++){
				double* feat1st_ = &feat1st[l*dim1st_per_rvec];

				/*Displacement*/
				int dx = x+rvecs[l];
				int dy = y+rvecs[l+nr];
				if( dy >= 0 && dy < height && dx >= 0 && dx < width ){
					int j = dy+dx*height;
					
					/* Gradient Magnitude */
					double mag2 = idata[j*3];
					if( mag2 < 0) continue; //tiny edge is ignored

					/*- Orientation index -*/
					int ind2  = (int)idata[j*3+1];
					int ind2n = (ind2+1)%nbin; //next bin

					/*- Weight for the index -*/
					double wei2  = idata[j*3+2];
					double wei2n = 1-wei2; //weight for the next bin

					/*- Weight for the pair -*/
					double mag = MIN(mag1, mag2);

					double mx = 0.5*(x+dx);
					double my = 0.5*(y+dy);

					/*- x-pos in spatial bins -*/
					double    rcx = pixel2wbin*mx;
					int     bincx = (int)floor(rcx-0.5);
					double rbincx = rcx - 0.5 - bincx;

					/*- y-pos in spatial bins -*/
					double    rcy = pixel2hbin*my;
					int     bincy = (int)floor(rcy-0.5);
					double rbincy = rcy - 0.5 - bincy;

					/*- Position weighting -*/
					double wc = exp( -gamma*((mx-centerx)*(mx-centerx)+(my-centery)*(my-centery)) );
					
					/*- Trilinear voting -*/
					for(int ix = 0; ix < 2; ix++)
						for(int iy = 0; iy < 2; iy++)
							if( bincx + ix >= 0 && bincx + ix < nwbin && bincy + iy >= 0 && bincy + iy < nhbin ){
								double weight = wc * mag * ABS(1 - ix - rbincx) * ABS(1 - iy - rbincy);
								int offset = dim1st * (bincx+ix+(bincy+iy)*nwbin);
								feat1st_[offset+ ind1  + ind2  * nbin]  += weight * wei1  * wei2;
								feat1st_[offset+ ind1n + ind2  * nbin]  += weight * wei1n * wei2;
								feat1st_[offset+ ind1  + ind2n * nbin] += weight * wei1  * wei2n;
								feat1st_[offset+ ind1n + ind2n * nbin] += weight * wei1n * wei2n;
							}
				}
			}
		}
	}
}
#endif
