/*=====================================================================*
 *
 * GLAC feature extraction on spatial bins
 *
 * feat = glac_on_grid_mex(Grad, rvecs, ztol, nbin, reflect, nsbins, gamma)
 *
 * Input
 *  Grad    - Gradient data matrix [height x width x 2(dx,dy)]
 *  rvecs   - Displacement (rx,ry) [nr x 2]
 *  ztol    - Tolerance for zero gradients [scalar] 
 *  nbin    - Number of orientation bins   [scalar]
 *  reflect - 1 for calculating direction in half-region (0~180) [1|0]
 *            0 for whole-region (0~360)
 *  nsbins  - Number of spatial bins (nwbin, nhbin)  [1 x 2]
 *  gamma   - Parameter of position weight, exp[-gamma*{(x-mu_x)^2+(y-mu_y)^2}]
 *
 * Output
 *  feat - Feature vector [nbin*nwbin*nhbin + nr*(nbin^2*nwbin*nhbin) x 1]
 *
 *=====================================================================*/
#include "mex.h"
#include "glac.hpp"

void mexFunction( int nlhs, mxArray *plhs[], 
		  int nrhs, const mxArray*prhs[] )
{ 
    if (nrhs != 7)      mexErrMsgTxt("glac_on_grid_mex: Wrong number of inputs."); 
    else if (nlhs != 1) mexErrMsgTxt("glac_on_grid_mex: Wrong number of outputs."); 
    
    /*-- Check the dimensions --*/ 
    if (!mxIsDouble(prhs[0]))                  mexErrMsgTxt("glac_mex: wrong data type of gradient matrix."); 
    if (mxGetNumberOfDimensions(prhs[0]) != 3) mexErrMsgTxt("glac_mex: wrong size of gradient matrix.");
	const mwSize* dim = mxGetDimensions(prhs[0]);
    int height = (int)dim[0];
    int width  = (int)dim[1];
	int depth  = (int)dim[2];
    if (depth != 2) mexErrMsgTxt("glac_mex: wrong size of gradient matrix");
	
	/*-- Gradient --*/
    double *grad;
    grad = (double*) mxGetPr(prhs[0]);


	/*-- Parameters --*/
	/*-Displacement vectors-*/
	if(!mxIsDouble(prhs[1]))                  mexErrMsgTxt("glac_on_grid_mex: wrong data type of displacement vectors."); 
	if(mxGetNumberOfDimensions(prhs[1]) != 2) mexErrMsgTxt("glac_on_grid_mex: wrong size of displacement vectors.");
	int     nr    = (int)mxGetM(prhs[1]);
	if (mxGetNumberOfElements(prhs[1]) != nr*2) mexErrMsgTxt("glac_on_grid_mex: wrong size of displacement vectors.");
	double* rvecs_= (double*)mxGetPr(prhs[1]);
	int*    rvecs = (int*)mxCalloc(2*nr, sizeof(int));
	for(int i = 0; i < 2*nr; i++) rvecs[i] = (int)rvecs_[i];

	/*-Zero tolerance-*/
	double ztol = mxGetScalar(prhs[2]);

	/*-Orientation bin-*/
	int nbin = (int)mxGetScalar(prhs[3]);

	/*-Orientation reflect-*/
	int reflect = (int)mxGetScalar(prhs[4]); //directional region to be quantized
	
	/*-Spatial bins-*/
	if (mxGetNumberOfElements(prhs[5]) != 2) mexErrMsgTxt("glac_on_grid_mex: wrong size of spatial bins.");
	double* sbins = (double*)mxGetPr(prhs[5]);
	int nwbin = (int)sbins[0];
	int nhbin = (int)sbins[1];

	/*-Gamma-*/
	double gamma = mxGetScalar(prhs[6]);


	/*-- Orientation coding --*/	
	double *codes = (double*)mxCalloc(width*height*3,sizeof(double));
	orientation_code(codes, grad, width*height, nbin, ztol, reflect);

	
	/*-- GLAC feature extraction on grid --*/
	plhs[0] = mxCreateDoubleMatrix(nbin*nwbin*nhbin+nr*nbin*nbin*nwbin*nhbin, 1, mxREAL); 
	double *feat = (double*)mxGetPr(plhs[0]);
	calc_glac_on_grid(feat, codes, height, width, nbin, rvecs, nr, nhbin, nwbin, gamma);

	mxFree(rvecs);
	mxFree(codes);

	return;
}
